#!/bin/bash

LOGFILE=/tmp/system.log
MAXSIZE=1048576

if [ -x /usr/local/bin/beroconf ]; then
	BEROCONF=/usr/local/bin/beroconf
else
	BEROCONF=/usr/fallback/beroconf
fi


# function definitions
function log {

	timestamp=$(date)
	logtype=${1}
	message=${2}

	if [ ! -z "${message}" ]; then
		echo "[${timestamp}] ${logtype}: ${message}" >> ${LOGFILE}
	fi
}

function eval_res {

	if [ ${1} -eq 0 ]; then
		echo "Success"
	else
		echo "Failure"
	fi
}

function log_services {

	service_list="lighttpd sshd isgw beroAvahi watchdog-service netconfigserver"
	process_list=$(/bin/ps)

	for service in ${service_list}; do
		echo "${process_list}" | grep ${service} > /dev/null
		if [ ${?} -eq 0 ]; then
			log "DAEMON" "service ${service} is running"
		else
			log "DAEMON" "service ${service} is not running"
		fi
	done

}

function log_ip {

	bc_ip=$(${BEROCONF} get root lan-ipaddr | grep -v failed)
	bc_nm=$(${BEROCONF} get root lan-netmask | grep -v failed)
	bc_gw=$(${BEROCONF} get root lan-gateway | grep -v failed)
	bc_ns=$(${BEROCONF} get root lan-nameserver | grep -v failed)

	log "NETWORK" "Configuration IP: ${bc_ip}/${bc_nm} GW: ${bc_gw} DNS: ${bc_ns}"

	if [ ! -z "${bc_gw}" ]; then
		/bin/ping -c 1 ${bc_gw} > /dev/null
		log "NETWORK" "result of ping to gateway (${bc_gw}): $(eval_res ${?})"
	else
		log "NETWORK" "no gateway configured"
	fi

	if [ ! -z "${bc_ns}" ]; then
		/bin/ping -c 1 ${bc_ns} > /dev/null
		log "NETWORK" "result of ping to nameserver (${bc_ns}): $(eval_res ${?})"

		/usr/bin/nslookup beronet.com > /dev/null
		log "NETWORK" "result of resolving beronet.com: $(eval_res ${?})"

		/bin/ping -c 1 beronet.com > /dev/null
		log "NETWORK" "result of ping to beronet.com: $(eval_res ${?})"
	else
		log "NETWORK" "no nameserver configured"

		/bin/ping -c 1 80.244.243.38 > /dev/null
		log "NETWORK" "result of ping to 80.244.243.38 (beronet.com): $(eval_res ${?})"
	fi
}

function log_mail {

	use_ssl=$(${BEROCONF} get root smgw_use_ssl | grep -v failed)

	# test POP3
	pop3_host=$(${BEROCONF} get root pop3_host | grep -v failed)
	pop3_port=$(${BEROCONF} get root pop3_port | grep -v failed)
	pop3_user=$(${BEROCONF} get root pop3_user | grep -v failed)
	pop3_pass=$(${BEROCONF} get root pop3_pass | grep -v failed)

	if [ -z "${pop3_host}" ] || [ -z "${pop3_user}" ] || [ -z "${pop3_pass}" ]; then
		log "MAIL" "POP3 host, user and/or password empty, not testing POP3 functionality."
	else
		pop3_args="-T -h ${pop3_host} -u ${pop3_user} -x ${pop3_pass}"

		if [ "${use_ssl}" = "1" ]; then
			pop3_args="${pop3_args} --use-ssl"
		fi

		log_text="result of connecting to ${pop3_host}"

		if [ ! -z "${pop3_port}" ]; then
			pop3_args="${pop3_args} --p ${pop3_args}"
			log_text="${log_text}:${pop3_port}"
		else
			log_text="${log_text}:25"
		fi

		/usr/local/bin/beroReadMail ${pop3_args} > /dev/null
		log "MAIL" "${log_text} as ${pop3_user} via POP3: $(eval_res ${?})"
	fi

	# test SMTP
	smtp_host=$(${BEROCONF} get root smtp_host | grep -v failed)
	smtp_port=$(${BEROCONF} get root smtp_port | grep -v failed)
	smtp_user=$(${BEROCONF} get root smtp_user | grep -v failed)
	smtp_pass=$(${BEROCONF} get root smtp_pass | grep -v failed)

	if [ -z "${smtp_host}" ] || [ -z "${smtp_user}" ] || [ -z "${smtp_pass}" ]; then
		log "MAIL" "SMTP host, user and/or password empty, not testing SMTP functionality."
	else
		smtp_args="--test --smtp-host ${smtp_host} --smtp-user ${smtp_user} --smtp-pass ${smtp_pass}"

		if [ "${use_ssl}" = "1" ]; then
			smtp_args="${smtp_args} --use-ssl"
		fi

		log_text="result of connecting to ${smtp_host}"

		if [ ! -z "${smtp_port}" ]; then
			smtp_args="${smtp_args} --smtp-port ${smtp_port}"
			log_text="${log_text}:${smtp_port}"
		else
			log_text="${log_text}:25"
		fi

		/usr/local/bin/beroSendMail ${smtp_args} > /dev/null
		log "MAIL" "${log_text} as ${smtp_user} via SMTP: $(eval_res ${?})"
	fi
}

function log_sip {

	sip_list=$(sed -n '/\[.*\]/ {N
			/\naddress/ p}' /usr/conf/isgw.sip)

	sip_list="${sip_list//[/|[}"

	while IFS='|' read -ra sip_arr; do
		for sip_item in "${sip_arr[@]}"; do
			if [ ! -z "${sip_item}" ]; then
				name=$(expr match "${sip_item}" ".*\[\(.*\)\]")
				addr=$(expr match "${sip_item}" ".*address = \(.*\)")
				/bin/ping -c 1 ${addr} > /dev/null
				log "SIP" "result of ping to SIP server ${name} (${addr}): $(eval_res ${?})"
			fi
		done
	done <<< ${sip_list}

	sip_reg=$(/usr/local/sbin/query_isgw r | tr -d '\r' | sed -e :a -e '$!N;s/\n/|/;ta')

	while IFS='|' read -ra r_arr; do
		for r_item in "${r_arr[@]}"; do
			if [ ! -z "${r_item}" ]; then
				log "SIP" "${r_item}"
			fi
		done
	done <<< ${sip_reg}
}

function log_port {

	port_state_i=$(/usr/local/sbin/query_isgw i | tr -d '\r' | sed -e :a -e '$!N;s/\n/|/;ta')
	sleep 1
	port_state_a=$(/usr/local/sbin/query_isgw a | tr -d '\r' | sed -e :a -e '$!N;s/\n/|/;ta')
	sleep 1
	port_state_g=$(/usr/local/sbin/query_isgw g | tr -d '\r' | sed -e :a -e '$!N;s/\n/|/;ta')

	while IFS='|' read -ra i_arr; do
		for i_item in "${i_arr[@]}"; do
			if [ ! -z "${i_item}" ]; then
				log "PORT" "${i_item}"
			fi
		done
	done <<< ${port_state_i}

	while IFS='|' read -ra a_arr; do
		for a_item in "${a_arr[@]}"; do
			if [ ! -z "${a_item}" ]; then
				log "PORT" "${a_item}"
			fi
		done
	done <<< ${port_state_a}

	while IFS='|' read -ra g_arr; do
		for g_item in "${g_arr[@]}"; do
			if [ ! -z "${g_item}" ]; then
				log "PORT" "${g_item}"
			fi
		done
	done <<< ${port_state_g}
}

function append_isgw {

	isgw_log=/tmp/isgw.err

	if [ -f ${isgw_log} ]; then
		isgw_err_cont=$(cat ${isgw_log} | sed -e :a -e '$!N;s/\n/|/;ta')

		while IFS='|' read -ra i_arr; do
			for i_item in "${i_arr[@]}"; do
				if [ ! -z "${i_item}" ]; then
					log "ISGW" "${i_item}"
				fi
			done
		done <<< ${isgw_err_cont}

		rm -f ${isgw_log}
	fi
}

function append_smgw {

	smgw_log=/tmp/smgw.err

	if [ -f ${smgw_log} ]; then
		isgw_err_cont=$(cat ${smgw_log} | sed -e :a -e '$!N;s/\n/|/;ta')

		while IFS='|' read -ra i_arr; do
			for i_item in "${i_arr[@]}"; do
				if [ ! -z "${i_item}" ]; then
					log "SMGW" "${i_item}"
				fi
			done
		done <<< ${isgw_err_cont}

		rm -f ${smgw_log}
	fi
}


# run functions

if [ "${1}" = "init" ]; then
	rm -f ${LOGFILE}
else
	if [ -f ${LOGFILE} ]; then
		cur_size=$(/bin/ls -l ${LOGFILE} | /usr/bin/awk '{ print $5 }')
	fi

	if [ -z ${cur_size} ]; then
		cur_size=0
	fi

	if [ ${cur_size} -ge ${MAXSIZE} ]; then
		mv ${LOGFILE} ${LOGFILE}.0
	fi
fi

log_services
log_ip
log_mail
log_sip
log_port
append_isgw
append_smgw

